@@ -44,7 +44,7 @@ def minimizing_delay(print_fixed_time_schedule: bool = False):
     logging.info(f"Loaded intersection and traffic situation from disk")
 
     logging.info(f"Minimizing delay")
-    fixed_time_schedule, phase_diagram, objective_value = SwiftMobilityCloudApi.get_optimized_fts(
+    fixed_time_schedule, phase_diagram, objective_value, _ = SwiftMobilityCloudApi.get_optimized_fts(
         intersection=intersection, arrival_rates=arrival_rates, min_period_duration=30, max_period_duration=180,
         objective=ObjectiveEnum.min_delay, horizon=2)
 
@@ -66,7 +66,7 @@ def minimizing_delay(print_fixed_time_schedule: bool = False):
     initial_queue_lengths = QueueLengths(id_to_queue_lengths=id_to_queue_lengths)
 
     logging.info(f"Minimizing delay")
-    fixed_time_schedule, phase_diagram, objective_value = SwiftMobilityCloudApi.get_optimized_fts(
+    fixed_time_schedule, phase_diagram, objective_value, _ = SwiftMobilityCloudApi.get_optimized_fts(
         intersection=intersection, arrival_rates=arrival_rates, min_period_duration=30, max_period_duration=180,
         objective=ObjectiveEnum.min_delay, horizon=2, initial_queue_lengths=initial_queue_lengths)
 
