@@ -7,7 +7,8 @@ from swift_cloud_py.entities.intersection.intersection import Intersection
 from swift_cloud_py.entities.intersection.traffic_light import TrafficLight
 from swift_cloud_py.entities.intersection.signalgroup import SignalGroup
 from swift_cloud_py.entities.control_output.fixed_time_schedule import FixedTimeSchedule, GreenYellowInterval
-from swift_cloud_py.entities.intersection.sg_relations import SyncStart, Offset, GreenyellowLead, Conflict
+from swift_cloud_py.entities.intersection.sg_relations import SyncStart, Offset, GreenyellowLead, Conflict, \
+    GreenyellowTrail
 from swift_cloud_py.validate_safety_restrictions.validate_other_sg_relations import find_other_sg_relation_matches, \
     get_shift_of_one_to_one_match, get_other_sg_relation_shift, validate_other_sg_relations
 
@@ -196,13 +197,15 @@ class TestFTSOtherSGRelationValidation(unittest.TestCase):
                                  sync_starts: List[SyncStart] = None,
                                  offsets: List[Offset] = None,
                                  greenyellow_leads: List[GreenyellowLead] = None,
+                                 greenyellow_trails: List[GreenyellowTrail] = None,
                                  ) -> Intersection:
         """
         Get a default intersection object with 2 conflicting signal groups "sg1" and "sg2"
         :param additional_signalgroups: signal groups to add to the intersection (besides signal group 'sg1' and 'sg2')
         :param sync_starts: SyncStarts that must be satisfied
         :param offsets: Coordinations that must be satisfied
-        :param greenyellow_leads: PreStarts that must be satisfied
+        :param greenyellow_leads: GreenyellowLeads that must be satisfied
+        :param greenyellow_trails: GreenyellowTrails that must be satisfied
         :return: the intersection object
         """
         if additional_signalgroups is None:
@@ -214,6 +217,8 @@ class TestFTSOtherSGRelationValidation(unittest.TestCase):
             offsets = []
         if greenyellow_leads is None:
             greenyellow_leads = []
+        if greenyellow_trails is None:
+            greenyellow_trails = []
 
         signalgroup1 = TestFTSOtherSGRelationValidation.get_default_signalgroup(name="sg1")
         signalgroup2 = TestFTSOtherSGRelationValidation.get_default_signalgroup(name="sg2")
@@ -222,7 +227,7 @@ class TestFTSOtherSGRelationValidation(unittest.TestCase):
 
         intersection = Intersection(signalgroups=[signalgroup1, signalgroup2] + additional_signalgroups,
                                     conflicts=[conflict], sync_starts=sync_starts, offsets=offsets,
-                                    greenyellow_leads=greenyellow_leads)
+                                    greenyellow_leads=greenyellow_leads, greenyellow_trails=greenyellow_trails)
 
         return intersection
 
@@ -405,13 +410,13 @@ class TestFTSOtherSGRelationValidation(unittest.TestCase):
         intersection = TestFTSOtherSGRelationValidation.get_default_intersection(
             additional_signalgroups=[signalgroup3, signalgroup4], greenyellow_leads=[greenyellow_lead])
 
-        for pre_start_time in [min_greenyellow_lead - 1, max_greenyellow_lead + 1]:
+        for lead_time in [min_greenyellow_lead - 1, max_greenyellow_lead + 1]:
             for interval_shift in range(2):
                 fts_copy = deepcopy(fts)
 
                 # adjust schedule to the specified greenyellow-lead
                 for index, greenyellow_interval in enumerate(fts_copy.get_greenyellow_intervals(signalgroup4)):
-                    greenyellow_start = (greenyellow_interval.start_greenyellow - pre_start_time) % fts_copy.period
+                    greenyellow_start = (greenyellow_interval.start_greenyellow - lead_time) % fts_copy.period
                     greenyellow_end = (greenyellow_start + 20) % fts_copy.period
                     fts_copy._greenyellow_intervals["sg3"][index] = GreenYellowInterval(
                         start_greenyellow=greenyellow_start, end_greenyellow=greenyellow_end)
@@ -419,7 +424,94 @@ class TestFTSOtherSGRelationValidation(unittest.TestCase):
                 fts_copy._greenyellow_intervals["sg4"] = fts_copy._greenyellow_intervals["sg4"][:interval_shift] + \
                     fts_copy._greenyellow_intervals["sg4"][interval_shift:]
 
-                with self.subTest(f"greenyellow_lead={pre_start_time}, interval_shift={interval_shift}"):
+                with self.subTest(f"greenyellow_lead={lead_time}, interval_shift={interval_shift}"):
+                    with self.assertRaises(SafetyViolation):
+                        # WHEN validating
+                        validate_other_sg_relations(intersection=intersection, fts=fts_copy)
+
+                        # THEN a SafetyViolation should be raised
+
+    def test_correct_greenyellow_trail(self) -> None:
+        """
+        Test that validation of correct greenyellow-lead passes.
+        :return:
+        """
+        # GIVEN
+        min_greenyellow_lead = 20
+        max_greenyellow_lead = 30
+        greenyellow_trail = GreenyellowTrail(from_id="sg3", to_id="sg4",
+                                             min_seconds=min_greenyellow_lead, max_seconds=max_greenyellow_lead)
+        fts = FixedTimeSchedule(greenyellow_intervals=dict(
+            sg1=[GreenYellowInterval(start_greenyellow=15, end_greenyellow=35)],
+            sg2=[GreenYellowInterval(start_greenyellow=45, end_greenyellow=65)],
+            sg3=[GreenYellowInterval(start_greenyellow=10, end_greenyellow=40),
+                 GreenYellowInterval(start_greenyellow=50, end_greenyellow=70)],
+            sg4=[GreenYellowInterval(start_greenyellow=30, end_greenyellow=40),
+                 GreenYellowInterval(start_greenyellow=69, end_greenyellow=90)]),
+            period=100)
+        signalgroup3 = TestFTSOtherSGRelationValidation.get_default_signalgroup(name="sg3")
+        signalgroup4 = TestFTSOtherSGRelationValidation.get_default_signalgroup(name="sg4")
+        intersection = TestFTSOtherSGRelationValidation.get_default_intersection(
+            additional_signalgroups=[signalgroup3, signalgroup4], greenyellow_trails=[greenyellow_trail])
+
+        for trail_time in [min_greenyellow_lead, max_greenyellow_lead]:
+            for interval_shift in range(2):
+                fts_copy = deepcopy(fts)
+
+                # adjust schedule to the specified greenyellow-trail
+                for index, greenyellow_interval in enumerate(fts_copy.get_greenyellow_intervals(signalgroup4)):
+                    greenyellow_end = (greenyellow_interval.end_greenyellow - trail_time) % fts_copy.period
+                    greenyellow_start = (greenyellow_end - 20) % fts_copy.period
+                    fts_copy._greenyellow_intervals["sg3"][index] = GreenYellowInterval(
+                        start_greenyellow=greenyellow_start, end_greenyellow=greenyellow_end)
+
+                fts_copy._greenyellow_intervals["sg4"] = fts_copy._greenyellow_intervals["sg4"][:interval_shift] + \
+                    fts_copy._greenyellow_intervals["sg4"][interval_shift:]
+
+                with self.subTest(f"greenyellow_lead={trail_time}, interval_shift={interval_shift}"):
+                    # WHEN validating
+                    validate_other_sg_relations(intersection=intersection, fts=fts_copy)
+
+                    # THEN no error should be raised
+
+    def test_incorrect_greenyellow_trail(self) -> None:
+        """
+        Test that validation of incorrect greenyellow-trail raises SafetyViolation.
+        :return:
+        """
+        # GIVEN
+        min_greenyellow_trail = 20
+        max_greenyellow_trail = 30
+        greenyellow_lead = GreenyellowLead(from_id="sg3", to_id="sg4",
+                                           min_seconds=min_greenyellow_trail, max_seconds=max_greenyellow_trail)
+        fts = FixedTimeSchedule(greenyellow_intervals=dict(
+            sg1=[GreenYellowInterval(start_greenyellow=15, end_greenyellow=35)],
+            sg2=[GreenYellowInterval(start_greenyellow=45, end_greenyellow=65)],
+            sg3=[GreenYellowInterval(start_greenyellow=10, end_greenyellow=40),
+                 GreenYellowInterval(start_greenyellow=50, end_greenyellow=70)],
+            sg4=[GreenYellowInterval(start_greenyellow=30, end_greenyellow=40),
+                 GreenYellowInterval(start_greenyellow=69, end_greenyellow=90)]),
+            period=100)
+        signalgroup3 = TestFTSOtherSGRelationValidation.get_default_signalgroup(name="sg3")
+        signalgroup4 = TestFTSOtherSGRelationValidation.get_default_signalgroup(name="sg4")
+        intersection = TestFTSOtherSGRelationValidation.get_default_intersection(
+            additional_signalgroups=[signalgroup3, signalgroup4], greenyellow_leads=[greenyellow_lead])
+
+        for trail_time in [min_greenyellow_trail - 1, max_greenyellow_trail + 1]:
+            for interval_shift in range(2):
+                fts_copy = deepcopy(fts)
+
+                # adjust schedule to the specified greenyellow-lead
+                for index, greenyellow_interval in enumerate(fts_copy.get_greenyellow_intervals(signalgroup4)):
+                    greenyellow_end = (greenyellow_interval.end_greenyellow - trail_time) % fts_copy.period
+                    greenyellow_start = (greenyellow_end - 20) % fts_copy.period
+                    fts_copy._greenyellow_intervals["sg3"][index] = GreenYellowInterval(
+                        start_greenyellow=greenyellow_start, end_greenyellow=greenyellow_end)
+
+                fts_copy._greenyellow_intervals["sg4"] = fts_copy._greenyellow_intervals["sg4"][:interval_shift] + \
+                    fts_copy._greenyellow_intervals["sg4"][interval_shift:]
+
+                with self.subTest(f"greenyellow_lead={trail_time}, interval_shift={interval_shift}"):
                     with self.assertRaises(SafetyViolation):
                         # WHEN validating
                         validate_other_sg_relations(intersection=intersection, fts=fts_copy)
